#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import Gradient as fg
from roller_constant_key import Option as ok
from roller_one import Hat
from roller_constant_fu import Fu
from roller_one_fu import Lay
from roller_render_hub import RenderHub
import gimpfu as fu

gf = Fu.GradientFill
pdb = fu.pdb


class GradientFill:
    """Fill the backdrop with a gradient."""

    @staticmethod
    def do(one):
        """
        Do Gradient Fill on a layer.

        image: GIMP image
            work-in-progress

        Return: layer or None
            with gradient material
        """
        z = GradientFill.do_layer(one)
        if z:
            Lay.clone(one.z)
            return Lay.merge(z)

    @staticmethod
    def do_layer(one):
        """
        Do Gradient Fill on a layer.

        Return: layer or None
            with gradient material
        """
        cat = Hat.cat
        j = cat.render.image
        if one.d[ok.OPACITY]:
            z = Lay.add(j, "Fill")
            z = RenderHub.do_rotated_layer(z, one.d, GradientFill.do_job)
            z.mode, z.opacity = RenderHub.get_mode(one.d)
            return RenderHub.bump(z, one.d[ok.BUMP])

    @staticmethod
    def do_job(z, d):
        """
        Draw a gradient.

        z: layer
            to receive gradient

        d: dict
            Has options.

        Return: layer
            with gradient
        """
        # Shape-burst methods require an opaque layer to work properly:
        j = z.image
        s = Hat.cat.render.size
        x, y, x1, y1 = RenderHub.get_layer_points(d, s[0], s[1])

        if d[ok.GRADIENT_TYPE] in fg.SHAPE_BURST:
            pdb.gimp_selection_all(j)
        return GradientFill.do_gradient(z, d, x, y, x1, y1)

    @staticmethod
    def do_gradient(z, d, x, y, x1, y1):
        """
        Apply a gradient.

        z: layer
            to receive gradient

        d: dict
            Has options.

        x, y: int
            start point

        x1, y1: int
            end point
        """
        RenderHub.set_fill_context(fg.FILL_DICT)
        pdb.gimp_context_set_gradient_blend_color_space(
            fu.GRADIENT_BLEND_RGB_PERCEPTUAL
        )
        pdb.gimp_context_set_gradient(d[ok.GRADIENT])
        pdb.gimp_context_set_gradient_reverse(d[ok.REVERSE])
        pdb.gimp_drawable_edit_gradient_fill(
            z,
            fg.GRADIENT_TYPE_LIST.index(d[ok.GRADIENT_TYPE]),
            d[ok.OFFSET],
            gf.YES_SUPERSAMPLE,
            gf.SUPERSAMPLE_MAX_DEPTH_2,
            gf.SUPERSAMPLE_THRESHOLD_0,
            gf.YES_DITHER,
            x, y,
            x1, y1
        )

        if d[ok.INVERT]:
            pdb.gimp_drawable_invert(z, 0)
        return z
