#!/usr/bin/env python
# -*- coding: utf-8 -*-
from random import randint, seed, uniform
from roller_constant_fu import Fu
from roller_constant_key import Option as ok
from roller_one import Base, Hat, One
from roller_one_fu import Lay
from roller_render_hub import RenderHub
from roller_render_shadow import Shadow
import gimpfu as fu
import math

pdb = fu.pdb

ONE_PIXEL = 1.
PI = 6.28318

# vine shadow dict:
DROP_SHADOW = {
    ok.SHADOW_BLUR: 20,
    ok.INTENSITY: 100,
    ok.OFFSET_X: -6,
    ok.OFFSET_Y: 0,
    ok.SHADOW_COLOR: (0, 0, 0),
    ok.MAKE_OPAQUE: 0
}


def plot_wave(h, start_x, amplitude, wave_length):
    """
    Calculate the points for a wave that starts
    at the top of image and ends at the bottom.

    h: int
        height of wave

    start_x: int
        x-coordinate to start wave

    amplitude: numeric
        in pixels
        of wave

    wave_length: numeric
        in pixels
    """
    q = []
    y = 0
    angle = 0.
    f = PI / wave_length

    while y < h:
        x = math.sin(angle) * amplitude + start_x
        q += [x, y]
        y += 1
        angle += f
    return q


class TrailingVine:
    """
    Create layers of lines that blur as they fade in the distance.
    """

    @staticmethod
    def do(one):
        """
        Do the Trailing Vine backdrop-style.

        one: One
            Has variables.

        Return: layer
            with Trailing Vine
        """
        d = one.d
        cat = Hat.cat
        j = cat.render.image
        z = Lay.clone(one.z)
        group = Lay.group(j, one.k)

        pdb.gimp_image_reorder_item(j, z, group, 0)

        n = "Edge"
        e = DROP_SHADOW
        w, h = cat.render.size
        span = min(w, h)
        stack = d[ok.LAYER_COUNT]
        shadow_blur_dec = 90 // stack

        # Preserve:
        foreground = pdb.gimp_context_get_foreground()

        seed(d[ok.RANDOM_SEED])
        RenderHub.set_brush_details()

        if d[ok.USE_PLASMA]:
            pdb.plug_in_plasma(
                j,
                z,
                d[ok.RANDOM_SEED],
                Fu.Plasma.LOWEST_TURBULENCE
            )
            Lay.blur(z, 500)

        for i in range(stack):
            amplitude = max(20, w // max(w, randint(span // 33, span // 11)))
            amp_inc = max(10, w // randint(50, 100))
            offset = h * uniform(-.5, .5)
            wave_length = h + offset
            z = Lay.add(j, n, parent=group)
            z.mode = fu.LAYER_MODE_DIFFERENCE
            e[ok.SHADOW_BLUR] = max(30, (90 - i * 10 - shadow_blur_dec))

            pdb.gimp_context_set_brush_size(ONE_PIXEL + i // 2)
            pdb.gimp_context_set_foreground(Base.rnd_col())

            for _ in range(d[ok.WAVE_PER_LAYER]):
                x = randint(0, w)
                q = plot_wave(h, x, amplitude, wave_length)

                pdb.gimp_paintbrush_default(z, len(q), q)
                amplitude += amp_inc

            z2 = Shadow.do(
                One(cast=(z,), model_name="", d=e, parent=group)
            )

            pdb.gimp_image_reorder_item(j, z2, group, 1)
            Lay.blur(z, (stack - i) * 3)

        z = Lay.merge_group(group)

        # Restore:
        pdb.gimp_context_set_foreground(foreground)

        return z
