#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import BackdropStyle as bs
from roller_constant_fu import Fu
from roller_constant_key import Option as ok
from roller_one import Hat
from roller_one_fu import Lay, Sel
from roller_render_gradient_light import GradientLight
from roller_render_hub import RenderHub
import gimpfu as fu

pdb = fu.pdb
um = Fu.UnsharpMask


def do_gradient(j, z, z1, color, color1, steps):
    """
    Do color blend effect using two colors.

    j: GIMP image
        work-in-progress

    z: layer
        to receive effect

    z1: layer
        with image

    color, color1: tuple
        RGB

    steps: int
        the number of steps to grow the
        selection to reach color1
    """
    step = RenderHub.calc_gradient(color, color1, steps)
    start_color = color
    q = list(start_color)
    a = None

    Sel.make_layer_sel(z1)

    for i in range(steps):
        if Sel.is_sel(j):
            a = pdb.gimp_selection_save(j)

            Sel.grow(j, 1, 0)
            Sel.load(j, a, option=fu.CHANNEL_OP_SUBTRACT)
            Sel.fill(z, tuple(q))
            Sel.load(j, a, option=fu.CHANNEL_OP_ADD)
            for x in range(3):
                q[x] = start_color[x] + int(step[x] * (i + 1))
        pdb.gimp_image_remove_channel(j, a)

    Sel.make_layer_sel(z1)
    Lay.clear_sel(z)


class GradientLevel:
    """Create a colored edge around image material."""

    @staticmethod
    def do(one):
        """
        Do the Gradient Bevel image-effect.
        Is an image-effect template function.

        one: One
            Has variables.
        """
        cat = Hat.cat
        j = cat.render.image
        d = one.d
        if d[ok.OPACITY]:
            parent = one.parent
            group = Lay.group(j, Lay.make_name(parent, one.k), parent=parent)
            z = Lay.add(j, one.k, parent=group)
            color, color1 = d[ok.COLOR_1], d[ok.COLOR_2]
            steps = d[ok.FRAME_WIDTH]
            n = d[ok.NOISE_MODE]

            do_gradient(j, z, one.image_layer, color, color1, steps)

            if n != "None" and d[ok.NOISE_OPACITY]:
                z1 = Lay.clone(z)
                z1.opacity = d[ok.NOISE_OPACITY]
                z1.mode = bs.NOISE_LAYER_MODE[bs.NOISE_MODE_LIST.index(n)]

                pdb.plug_in_plasma(
                    j, z1,
                    d[ok.RANDOM_SEED],
                    Fu.Plasma.LOWEST_TURBULENCE
                )
                Sel.item(z)
                Sel.invert_clear(z1)

                if n == bs.SYRUPY:
                    pdb.plug_in_unsharp_mask(
                        j, z1,
                        um.RADIUS_5,
                        um.AMOUNT_1,
                        um.THRESHOLD_0
                    )
                    pdb.gimp_drawable_desaturate(z1, fu.DESATURATE_LUMINANCE)
                elif n == bs.CARTOONY:
                    avg = tuple(
                        [(color[i] + color1[i]) // 2 for i in range(3)]
                    )
                    RenderHub.do_cartoony(z, z1, avg)

            z = Lay.merge_group(group)
            z.opacity = d[ok.OPACITY]
            z = GradientLight.apply_light(z, ok.TRANSLUCENT_FRAME)
            one.shadow_layer = [z, one.image_layer]
            return z
