#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import Widget as fw
from roller_constant_key import Widget as wk
from roller_option_preset import Preset
from roller_option_group import OptionGroup
from roller_port_preview import PortPreview
from roller_window_save import RWSave


class PortLight(PortPreview):
    """Is a port with Gradient Light influence slider widgets."""

    def __init__(self, d, g):
        """
        Start it up.

        d: dict
            Has init values.

        g: OptionButton
            Has option values.
        """
        PortPreview.__init__(self, d, g)

    def _draw_influence(self, g):
        """
        Draw widgets in a table.

        g: VBox
            container for widgets

        q: iterable
            of strings
            keys to widgets
        """
        key = self.safe.key
        k = self.safe.group.group_key
        d = OptionGroup.draw_group(
            **{
                wk.COLOR: self.color,
                wk.CONTAINER: g,
                wk.GROUP_KEY: key,
                wk.GROUP_TYPE: Preset,
                wk.HAS_PRESET: True,
                wk.IS_DEFAULT: False,
                wk.KEYS: Preset.get_keys(key),
                wk.ON_KEY_PRESS: self.on_key_press,
                wk.ON_PREVIEW_BUTTON: self.task_preview,
                wk.ON_WIDGET_CHANGE: self.on_widget_change,
                wk.PATH: (k,),
                wk.PORT: self,
                wk.SAVE_WINDOW: RWSave,
                wk.WIN: self.roller_window
            }
        )
        g = self.preset = d[wk.PRESET]
        self.group = g.group
        g.load_preset(self.safe.get_value(), fw.UNDEFINED)

    def draw_port(self, g):
        """
        Draw the port's widgets.

        Is part of the Port template.

        g: VBox
            container for widgets
        """
        self.draw_simple_dialog_port(
            g,
            (self._draw_influence, self.draw_preview_process),
            ("Set Influence", "")
        )

    def get_group_value(self):
        """
        Use to get the preset value.
        Call from PortPreview.

        Return: dict
            of gradient light influence
        """
        return self.preset.get_value()
