#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import Margin as fm, Widget as fw
from roller_constant_key import Option as ok, Path as pa, Widget as wk
from roller_one_draw import Draw
from roller_option_preset import Preset
from roller_option_group import OptionGroup
from roller_port_preview import PortPreview
from roller_window_save import RWSave


class PortMargin(PortPreview):
    """Is a port with fixed and factor margin widgets."""

    def __init__(self, d, g):
        """
        Start it up.

        d: dict
            Has init values.

        g: OptionButton
            Has option values.
        """
        PortPreview.__init__(self, d, g)

    def _draw_margins(self, g):
        """
        Draw widgets in a table.

        g: VBox
            container for widgets

        q: iterable
            of strings
            keys to widgets
        """
        k = self.safe.key
        path = self.safe.group.path[:3] + (k,)
        d = OptionGroup.draw_group(
            **{
                wk.COLOR: self.color,
                wk.CONTAINER: g,
                wk.GROUP_KEY: k,
                wk.GROUP_TYPE: Preset,
                wk.HAS_PRESET: True,
                wk.IS_DEFAULT: False,
                wk.KEYS: Preset.get_keys(k),
                wk.ON_KEY_PRESS: self.on_key_press,
                wk.ON_PREVIEW_BUTTON: self.task_preview,
                wk.ON_WIDGET_CHANGE: self.on_widget_change,
                wk.PATH: path,
                wk.PORT: self,
                wk.SAVE_WINDOW: RWSave,
                wk.WIN: self.roller_window
            }
        )
        g = self.preset = d[wk.PRESET]
        self.group = g.group
        a = self.safe.group
        k1 = pa.MARGIN + "," + a.group_key.split(",")[1]

        g.load_preset(self.safe.get_value(), fw.UNDEFINED)
        d[ok.MARGIN_SIZE].set_caption_type(k1, a.path)

    def draw_port(self, g):
        """
        Draw the port's widgets.

        Is part of the Port template.

        g: VBox
            container for widgets
        """
        self.draw_simple_dialog_port(
            g,
            (self._draw_margins, self.draw_preview_process),
            ("Set Margins", "")
        )

    def get_group_value(self):
        """
        Use to get the preset value.
        Call from PortPreview.

        Return: dict
            of margin
        """
        return self.preset.get_value()

    def on_widget_change(self, g):
        """
        Relay a change event from the sub-port widgets.
        Set the sensitivity of the Preview Button.

        g: Widget
            Is responsible for the change event.

        Return: Done or None
            from the super class
            for the GTK event handler
        """
        if not Draw.load_count:
            if g.key in fm.MARGIN_KEY or g.key == wk.PRESET:
                g.group.d[ok.MARGIN_SIZE].update_size()
        return super(PortMargin, self).on_widget_change(g)
