#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import Widget as fw
from roller_constant_key import (
    Button as bk,
    Effect as ek,
    Group as gk,
    Step as sk,
    Widget as wk
)
from roller_one import Hat
from roller_one_draw import Draw
from roller_option_preset import Preset, SuperPreset
from roller_port import Port
from roller_port_preview import PortPreview
from roller_widget_box import VBow
from roller_widget_label import Label
from roller_window_save import RWSave
import gtk

SHADOW_KEY = Preset.get_keys(ek.SHADOW_1)


class PortShadow(PortPreview):
    """Offer options for defining a shadow."""

    def __init__(self, d, g):
        """
        Create a shadow-choice port.

        d: dict
            Has init values.

        g: OptionButton
            Has option values.
        """
        self.d = {wk.SAVE_WINDOW: RWSave}

        d.update(self.d)
        PortPreview.__init__(self, d, g)

    def _draw_navigation(self, g):
        """
        Draw the shadow choices group.

        g: GTK container
            to receive group
        """
        self.d.update(
            {
                wk.COLOR: self.color,
                wk.IS_DEFAULT: False,
                wk.WIN: self.roller_window
            }
        )

        hbox = self.d[wk.NAV_BOX] = gtk.HBox()
        vbox = VBow()
        vbox.add(
            Label(
                padding=(4, 4, 4, fw.MARGIN),
                text="Shadows:"
            )
        )
        hbox.junctions = [vbox]
        a = self.color

        hbox.add(vbox)
        g.add(hbox)
        self._draw_options(vbox, gk.TRI_SHADOW, 0, ())

        self.color = a
        g = Hat.cat.group_dict[sk.TRI_SHADOW_PRESET].d[wk.PRESET]
        self.group = g.group

        g.load_preset(self.safe.get_value(), fw.UNDEFINED)
        for i in range(len(self.d[wk.NAV_BOX].junctions)):
            self.reduce_color()

    def draw_port(self, g):
        """
        Draw the port's widgets.

        Is part of the Port template.

        g: VBox
            container for the widgets
        """
        self.draw_simple_dialog_port(
            g,
            (self._draw_navigation, self.draw_preview_process),
            ("Shadow Options", "")
        )

    def get_group_value(self):
        """
        Use to get the preset value.
        Call from PortPreview.

        Return: dict
            of tri-shadow, the super-preset
        """
        return SuperPreset.get_steps(sk.TRI_SHADOW, with_list=False)[0]

    def on_widget_change(self, g):
        """
        Call when a widget value changes.

        g: Widget
            Is responsible.
        """
        if not Draw.load_count:
            if g.key == bk.RANDOM:
                Port.randomize_widgets(g)
        if g.group.preset:
            if g.key != wk.PRESET:
                g.group.preset.set_to_undefined()
        return super(PortShadow, self).on_widget_change(g)
