#!/usr/bin/env python
# -*- coding: utf-8 -*-
import gtk
from roller_constant_for import Color as co
from roller_constant_key import Widget as wk

MUST = {
    wk.ALIGN: (0, 0, 0, 0),
    wk.BOX: gtk.VBox,
    wk.PADDING: (0, 0, 0, 0)
}


class Box(gtk.Alignment):
    """This is a GTK Box widget with an Alignment."""

    def __init__(self, **d):
        """
        Create a box container for other widgets.

        keys:
            box_type: class
                either VBox, HBox, VButtonBox, HButtonBox

            align: tuple
                of float
                top, bottom, left, right
                Alignment
                in .0 to 1.

            padding: tuple of int
                top, bottom, left, right
        """
        super(gtk.Alignment, self).__init__()

        for k in MUST:
            a = d[k] if k in d else MUST[k]
            setattr(self, k, a)

        self.box = self.box()

        self.set(*self.align)
        self.set_padding(*self.padding)
        super(gtk.Alignment, self).add(self.box)

    def add(self, g):
        """
        Add a widget to the box.

        g: GTK widget
            to add
        """
        self.box.add(g)

    def get_children(self):
        return self.box.get_children()

    def pack_start(self, g, **d):
        """
        Place a widget in the box.

        g: GTK widget
            Is packed into container.

        d: dict
            keyword options
        """
        self.box.pack_start(g, **d)


class VBow(gtk.VBox):
    """
    Is a wrapper for the gtk VBox.

    Provide an object to store variables
    with a VBox.
    """

    def __init__(self):
        """Create the VBox."""
        super(gtk.VBox, self).__init__()


class Eventful(gtk.EventBox):
    """This is a custom GTK EventBox."""

    def __init__(self, color):
        """
        color: gtk color component
            None, int, tuple, or GTK color
        """
        super(gtk.EventBox, self).__init__()

        self.value = None
        if color is not None:
            if isinstance(color, int):
                q = gtk.gdk.Color(color, color, co.MAX_COLOR)

            elif isinstance(color, tuple):
                q = gtk.gdk.Color(*color)

            else:
                q = color
            self.modify_bg(gtk.STATE_NORMAL, q)

    def get_value(self):
        """
        Is part of the Widget template.
        Is used by the PortCell to create a 'safe'.

        Return: value
            from 'self.value'
        """
        return self.value

    def set_value(self, a):
        """
        Set 'self.value'.

        Is part of the Widget template.
        Is used by the PortCell to create a 'safe'.

        a: value
            Give to 'self.value'.
        """
        self.value = a
