#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import Color as co, Widget as fw
from roller_constant_key import Widget as wk
from roller_widget_box import Eventful
from roller_widget_label import Label
import gtk

LABEL_INDEX, WIDGET_INDEX, ARGS_INDEX, KEY_ARGS_INDEX = range(4)


class Table(gtk.Alignment):
    """Organize GTK table methods."""

    @staticmethod
    def create(**d):
        """
        Put widgets in a two column GTK table.

        Each widget must have a key if it is
        to be returned in the widget dict.

        d: dict
            Has keyword arguments.

        Return: dict
            of widget
        """
        w = fw.MARGIN
        w1 = w // 2
        bottom_pad = d[wk.BOTTOM_PAD] if wk.BOTTOM_PAD in d else w
        top_pad = d[wk.TOP_PAD] if wk.TOP_PAD in d else w
        color = d[wk.COLOR]
        q = d['q']
        e = {}
        alignment = gtk.Alignment(0, 0, 1, 0)

        alignment.set_padding(top_pad, bottom_pad, w1, w1)

        table = gtk.Table(len(q), 2)
        option_count = len(q)

        table.set_row_spacings(1)

        for x, i in enumerate(q):
            if not x:
                # Expand the right-column with an invisible label widget:
                table.attach(gtk.Label("\t" * 6), 1, 2, x, x + 1)

            g1 = i[WIDGET_INDEX](**i[ARGS_INDEX])
            color = Table.get_darker_color(color, option_count)
            color1 = color, color, co.MAX_COLOR
            label = Label(text=i[LABEL_INDEX])
            box = label.box = Eventful(color1)
            box1 = g1.box = Eventful(color1)
            g1.label = label

            if hasattr(g1, 'key'):
                e[g1.key] = g1

            label.set_padding(0, 0, w, w1)
            g1.set_padding(0, 0, w1, w)
            box.add(label)
            box1.add(g1)
            table.attach(box, 0, 1, x, x + 1)
            table.attach(box1, 1, 2, x, x + 1, yoptions=gtk.FILL)

        alignment.add(table)
        d[wk.CONTAINER].pack_start(alignment, expand=True)
        return e

    @staticmethod
    def get_darker_color(color, step_count):
        """
        Calculate a darker blue color with a step count.

        color: int
            to decrease
            Is the red and green component.

        step_count: int
            Use to compute amount to reduce.

        Return: int
            for a darker color
        """
        return color - 14000 // (step_count + 1)
