#!/usr/bin/env python
# -*- coding: utf-8 -*-
from copy import deepcopy
from roller_constant_key import (
    Pickle as pc,
    Preset as pk,
    Widget as wk,
    Window as wi
)
from roller_one import OZ, Comm, Hat
from roller_port_save import PortSave
from roller_window import Window
import os


class RWSave(Window):
    """Is a GTK dialog used to save a preset."""
    preset_name = "New Preset"

    DUPLICATE_FILE = \
        "The file,\n\n{},\n\nalready exists. " \
        "Do you want to overwrite the existing file?"

    def __init__(self, d):
        """
        Create a window.

        d: dict
            Has init values.
        """
        self._init_dict = d
        self.get_data = d[pk.GET_DATA]
        self.key = d['key']
        self.is_write = False
        n = d[wk.WINDOW_TITLE] = "Name the " + self.key + " Preset"
        d[wk.WINDOW_KEY] = wi.SAVE

        Window.__init__(self, d)

        e = {
            wk.ON_ACCEPT: self.accept,
            wk.ON_CANCEL: self.cancel,
            wk.SAVE_WINDOW: self,
            wk.WIN: self,
            wk.WINDOW_TITLE: n
        }
        self.port = PortSave(e, RWSave.preset_name, self.key)

        self.win.vbox.add(self.port.pane)
        self.win.show_all()
        self.win.run()
        self.win.destroy()

    def accept(self, n):
        """
        Write the preset, and if successful, close the window.

        n: string
            name of preset

        Return: flag
            If true, the window is done.
        """
        d = deepcopy(self.get_data())
        if RWSave.write(self.win, d, self.key, n):
            self._init_dict[pk.FILE_NAME] = RWSave.preset_name = n
            self.is_write = True
            return self.close()

    @staticmethod
    def write(win, d, key, file_name, overwrite=False):
        """
        Write the preset, after checking if it already
        exists and asking permission to over-write.

        d: dict
            Has preset data.

        file_name: string
            second part of the file path

        overwrite: flag
            If its false, a dialog will confirm a file over-write.

        Return: flag
            Is true if the file was written.
        """
        path = OZ.get_preset_path(
            key,
            file_name,
            Hat.cat.preset_folder
        )
        go = True

        if not overwrite:
            if path:
                if os.path.isfile(path):
                    n = RWSave.DUPLICATE_FILE.format(path)
                    go = Comm.pop_up(win, 0, n, "Duplicate File")

        if go:
            go = OZ.pickle_dump({pc.DATA: d, pc.FILE: path})
        return go
